/* global variables for mad program */

#define LMAR 50                    // left margin for y-axis
#define TICWIDTH 5                 // width of the tics of the y-axis
#define UPDATE_TIMER 5             // update of time display in sec.
#define VERSION "4.3"              // program version
#define DIRECTDAT  "out"           // name of output file for direct channels
#define DERIVEDDAT "derout"        // name of output file for derived channels
#define COMMENTDAT "comment.xmmad" // name of comment file
#include <fstream.h> 

typedef struct {
  int   ad_low, ad_high;
  float rw_low, rw_high;
} calib;

typedef struct {
  int   length;      // display length in sec;
  int   xpix;        // number of horizontal pixels in display;
  int   sr;          // sampling rate;
  int   recon;       // program is recording (1) or sleeping (0);
  int   nbuf;        // number of samples read from FIFO and scanned for minmax;
  int   nbuftrue;    // actual size of ch[i].buf, important when window is resized during recording;
  int   nactive;     // number of active channels;
  int   resize;      // window was resized (1);
  int   movavg;      // number of values for movavg;
  int   store;       // save data on file (1) or not (0);
  int   analog;      // show analog values (1) or not (0);
  int   needsave;    // need to save (1) or not (0);
} regist;

typedef struct {
  int   *min;         // array stores min of nbuf samples;
  int   *max;         // array stores max of nbuf samples
  char  *buf;         // buffer to read from FIFO;
  int   nread;        // number of values in buffer;
  int   first,last;   // first and last element of minmax array;
  short addfirst;     // whether first must be incremented;
  int   xpos;         // x position of last sample drawn on canvas;
  int   active;       // channel is active;
  int   polarity;     // uni (0) or bipolar (1) mode; 
  int   color;        // color of trace (0-7);
  int   height;       // relative height of channel display (0-3);
  calib cal;          // calibration factors;
  Widget analog;      // Widget to show analog value;
  unsigned long analog_sum;   // Sum of values for analog display;
  unsigned long analog_n;     // number of values added in analog_sum;
} channel;

/* Structure for derived channels */
typedef struct {
  int   nch;
  int   threshold;
  int   movavg;
  int   ampli;
  int   *mavg;
  unsigned long masum;	
  channel  ch[7]; /* 0:freq, 1:max, 2:mean, 3:min, 4:ampl, 5:dy/dtmin, 6:dy/dtmax; */
} der_channel;

/* Structure for menu system */
typedef struct {
  Widget f_menu;
  Widget l_setup;
  Widget s_setup;
  Widget s_data;
  Widget quit;
} f_menu;

typedef struct {
  Widget r_menu;
  Widget s_recording;
  Widget e_recording;
} r_menu;

typedef struct {
  Widget c_menu;
  Widget channel[8];
} c_menu;

typedef struct {
  Widget o_menu;
  Widget srate;
  Widget tracel;
  Widget achannel;
  Widget analog;
  Widget store;
} o_menu;

typedef struct {
  f_menu file;
  r_menu recording;
  c_menu calibrate;
  o_menu options;
} main_menu;

extern String colors[];

extern XtAppContext app;
extern XtWorkProcId work_id;
extern Widget       drawing_a, mess_label, trace_label, sr_label, comment_label, analog_w;

extern ofstream     datei[8], derdat[8], commentdat;
extern regist       reg, der;
extern channel      ch[8];
extern der_channel  dch[8];
extern main_menu    mymenu;
extern int          fd[8], fd8, fd9;
extern time_t       initial_zeit; /* time at which recording was started */
